@@ -295,61 +295,85 @@ def stop_experiment(id):
     producer.flush()
     print("Experiment stopped")
 
-def experiment_spawner():
-    experiments={}
-    exp_consumer = kafka_mdml_consumer(["mdml-experiment-service"], "mdml-experiment-service")
-    for msg in exp_consumer.consume(overall_timeout=-1):
-        print(msg)
-        if msg['value']['status'] == "on":
-            #Message will contain topics to include under an experiment
-            t = threading.Thread(target=experiment_task, args=(msg,))
-            experiments[msg['value']['experiment_id']] = t
-            t.setDaemon(True)
-            t.start()
+# def replay_experiment(exp_id, group, method, print_all_data=False, study_id=None, AD_SDL_token=None):
+#     data = []
+#     exp_consumer = kafka_mdml_consumer_schemaless([f"mdml-experiment-{exp_id}"], group)
+#     print("Gathering experiment data for replaying.")
+#     for msg in exp_consumer.consume(overall_timeout=5, verbose=False):
+#         data.append(json.loads(msg['value']))
+#     if len(data) == 0:
+#         print("No experiment data found. You must use a unique group ID for each replay.")
+#         return
+#     print("Calculating simulation timeline.")
+#     for d in data:
+#         d['time'] = d['value']['time']
+#     data = sorted(data, key=lambda k: k['time'])
+#     real_start_timestamp = data[0]['time']
+#     for d in data:
+#         d['time_diff'] = d['time'] - real_start_timestamp
+#     if print_all_data:
+#         print(data)
+#     print("Starting replay.")
+#     sim_start_timestamp = time.time()
+#     if method == "print":
+#         while len(data) > 0:
+#             if data[0]['time_diff'] <= time.time() - sim_start_timestamp:
+#                 print(data[0])
+#                 del data[0]
+#     elif method == "produce":
+#         print("Implement re-producer method")
 
-def experiment_task(msg):
-    exp_id = msg['value']['experiment_id']
-    topics = msg['value']['topics']
-    topics.append("mdml-experiment-service")
-    exp_producer = kafka_mdml_producer_schemaless(f"mdml-experiment-{exp_id}")
-    exp_consumer = kafka_mdml_consumer(topics, f"mdml-experiment-{exp_id}", auto_offset_reset="latest")
-    num_msgs = 0
-    for msg in exp_consumer.consume(overall_timeout=-1, verbose=False):
-        if msg['topic'] == "mdml-experiment-service":
-            if msg['value']['status'] == "off":
-                exp_producer.flush()
-                print("Stopping consumer loop...")
-                break
-        num_msgs += 1
-        exp_producer.produce(json.dumps(msg))
-        # Determine optimal strategy for flushing 
-        if num_msgs % 500 == 0:
-            exp_producer.flush()
+#     elif method == "upload":
+#         with open(f'{exp_id}.json', 'w') as f:
+#             f.writelines(json.dumps(data))
+#         if study_id is None or AD_SDL_token is None:
+#             Exception("cannot use method 'upload' without a study_id")
+#         else:
+#             from adc_sdk.client import ADCClient
+#             client = ADCClient(AD_SDL_token)
+#             with open(f'{exp_id}.json', 'rb') as f:
+#                 sample = client.create_sample(f,study_id,f"MDML experiment {exp_id}")
+#             os.remove(f"{exp_id}.json")
 
-def replay_experiment(exp_id, group, print_all_data=False):
-    data = []
-    exp_consumer = kafka_mdml_consumer_schemaless([f"mdml-experiment-{exp_id}"], group)
-    print("Gathering experiment data for replaying.")
-    for msg in exp_consumer.consume(overall_timeout=5, verbose=False):
-        data.append(json.loads(msg['value']))
-    if len(data) == 0:
-        print("No experiment data found. You must use a unique group ID for each replay.")
-        return
-    print("Calculating simulation timeline.")
-    for d in data:
-        d['time'] = d['value']['time']
-    data = sorted(data, key=lambda k: k['time'])
-    real_start_timestamp = data[0]['time']
-    for d in data:
-        d['time_diff'] = d['time'] - real_start_timestamp
-    if print_all_data:
-        print(data)
-    print("Starting replay.")
-    sim_start_timestamp = time.time()
-    while len(data) > 0:
-        if data[0]['time_diff'] <= time.time() - sim_start_timestamp:
-            print(data[0])
-            del data[0]
+def replay_experiment(id, group):
+    """
+    Create an experiment that writes all messages from the given topics to a new singular topic
+    
+    Parameters
+    ==========
+    id : str
+        Unique ID of the experiment to replay
+    group : str
+        group to use when consuming the main experiment topic
+    """
+    experiment_replay_schema = {
+        "$schema": "http://merf.egs.anl.gov/mdml-experiment-replay-schema#",
+        "title": "ExperimentReplaySchema",
+        "description": "Schema for Kafka MDML Experiment Replays",
+        "type": "object",
+        "properties": {
+            "time": {
+                "description": "Sent timestamp",
+                "type": "number"
+            },
+            "experiment_id": {
+                "description": "A unique experiment ID",
+                "type": "string"
+            },
+            "group": {
+                "description": "Group ID for the main experiment topic",
+                "type": "string"
+            }
+        },
+        "required": [ "time", "experiment_id", "group" ]
+    }
+    producer = kafka_mdml_producer("mdml-experiment-replay", schema=experiment_replay_schema)
+    producer.produce({
+        "time": time.time(),
+        "experiment_id": id,
+        "group": group
+    })
+    producer.flush()
 
 def create_schema(d, title, descr, required_keys=None):
     """
